package com.erikloyer.swing
{
	import flash.display.Sprite;
	import flash.text.*;

	/**
	 * HangingLetter, Version 1.
 	 * Handles a single hanging letter.
	 * 
	 * @author: Erik Loyer
	 * @version: 1.0.0
	 */
	public class HangingLetter extends Sprite
	{
		private var _rotAcc:Number = 0;										// rotational acceleration
		private var _friction:Number = .75 + (Math.random() * .15);			// friction applied to this letter
		private var _field:TextField = new TextField();						// the text field
		private var _format:TextFormat = new TextFormat();					// the text format
		
		/**
		 * Constructor
		 * 
		 * @param	char	The character we are displaying.
		 * @param	color	The color to make the character.
		 * @param	mute	Are we to mute this letter?
		 */
		public function HangingLetter(char:String, color:Number, mute:Boolean) {
			
			// set up title format
			_format.font = "Helvetica Neue";
			_format.bold = true;
			_format.size = 150;
			if (mute) {
				visible = false;
			}
			_format.color = color;
			_format.align = TextFormatAlign.LEFT;
			
			// set up title
			_field.embedFonts = true;
			_field.autoSize = TextFieldAutoSize.LEFT;
			_field.selectable = false;
			_field.text = char;
			_field.setTextFormat(_format);
			
			_field.x = - (_field.textWidth / 2);
			
			addChild(_field);
			
		}
		
		/**
		 * Per frame update to letter properties.
		 * 
		 * @param	inheritedRotAcc		Rotational acceleration inherited from the invisible bar we're attached to.
		 * @param	inheritedRotation	Rotation inherited from the invisible bar we're attached to.
		 */
		public function perFrame(inheritedRotAcc:Number, inheritedRotation:Number):void {
			_rotAcc += (-inheritedRotation - rotation) / 10;
			rotation += _rotAcc;
			_rotAcc *= _friction;
		}
		
		/**
		 * Updates the mute status of this letter.
		 */
		public function updateMuting(mute:Boolean):void {
			if (mute) {
				visible = false;
			} else {
				visible = true;
			}
		}
		
		/* -- getters -- */
		
		public function get textWidth():Number { return _field.textWidth; }
		
	}
}