package com.erikloyer.swing
{
	import flash.display.*;
	import flash.text.*;

	/**
	 * TextButton, Version 1.
	 * A text-based button with optional square bullet.
	 * 
	 * @author: Erik Loyer
	 * @version: 1.0.0
	 */
	public class TextButton extends SimpleButton
	{
		private var _hasBullet:Boolean;						// does the button have a bullet?
		private var _bulletSize:Number;						// the width/height of the bullet in pixels
		private var _labelText:String;						// text of the button
		private var _format:TextFormat;						// text format for button text
		private var _upSprite:Sprite;						// up state sprite
		private var _overSprite:Sprite;						// over state sprite
		private var _highlightSprite:Sprite;				// over state sprite
		private var _hitTestShape:Shape = new Shape;		// hit test shape
		private var _labelMargin:Number;					// amount of space between bullet and label
		private var _colors:Object;							// colors for the button
		
		/**
		 * Constructor
		 * 
		 * @param	labelText	The text of the button label.
		 * @param	format		Text format for the label.
		 * @param	colors		Contains colors for the button.
		 * @param	hasBullet	If true, a bullet will be displayed for the button.
		 * @param	bulletSize	The width/height of the bullet in pixels.
		 * @param	margin		The margin between the bullet and the text in pixels.
		 */
		public function TextButton(labelText:String, format:TextFormat, colors:Object, hasBullet:Boolean=false, bulletSize:Number=6, margin:Number=3)
		{
			_format = format;
			_labelText = labelText;
			_labelMargin = margin;
			_hasBullet = hasBullet;
			_bulletSize = bulletSize;
			_colors = colors;
			
			// create button state sprites
			_upSprite = createSprite(_colors.up);
			_overSprite = createSprite(_colors.over);
			_highlightSprite = createSprite(_colors.highlight);
			
			// set up the states
			upState = _upSprite;
			overState = _overSprite;
			downState = overState;
			
			// set up the hit test state
			_hitTestShape.graphics.beginFill(0x000000, 1);
			_hitTestShape.graphics.moveTo(0, 0);
			_hitTestShape.graphics.lineTo(_upSprite.width, 0);
			_hitTestShape.graphics.lineTo(_upSprite.width, _upSprite.height);
			_hitTestShape.graphics.lineTo(0, _upSprite.height);
			_hitTestShape.graphics.lineTo(0, 0);
			_hitTestShape.graphics.endFill();
			hitTestState = _hitTestShape;
			
		}
		
		/**
		 * Returns a sprite for the button in the given color.
		 * 
		 * @param	color	The color to make the sprite.
		 */
		private function createSprite(color:Number):Sprite {
			
			_format.color = color;
			var sprite:Sprite = new Sprite();
			
			// set up the label text field
			var label:TextField = new TextField();
			label.embedFonts = true;
			label.antiAliasType = AntiAliasType.ADVANCED;
			label.autoSize = TextFieldAutoSize.LEFT;
			label.selectable = false;
			label.text = _labelText;
			label.setTextFormat(_format);
			sprite.addChild(label);
			
			// if the button has a bullet
			if (_hasBullet) {
				var bullet:Shape = new Shape();
				bullet.graphics.beginFill(color, 1);
				bullet.graphics.drawRect(0, 0, _bulletSize, _bulletSize);
				bullet.graphics.endFill();
				bullet.y = (label.height - _bulletSize) / 2 + 1;
				sprite.addChild(bullet);
				label.x = _bulletSize + _labelMargin;
			}
			
			return sprite;
		}
		
		/**
		 * Highlights the button while in its up state.
		 */
		public function highlight():void {
			upState = _highlightSprite;
		}
		
		/**
		 * Unhighlights the button while in its up state.
		 */
		public function unhighlight():void {
			upState = _upSprite;
		}		
	}
}