package com.erikloyer.swing
{
	import flash.display.Sprite;
	import flash.text.*;
	import flash.geom.Rectangle;
	import flash.events.*;
	import org.wiiflash.events.ButtonEvent;

	/**
	 * TextInput, Version 1.
 	 * Allows the user to input their own text.
	 * 
	 * @author: Erik Loyer
	 * @version: 1.0.0
	 */
	public class TextInput extends Sprite
	{
		private var _field:TextField = new TextField();							// text entry field
		private var _format:TextFormat = new TextFormat();						// text entry format
		private var _instructions:TextField = new TextField();					// instructions field
		private var _instructionsFormat:TextFormat = new TextFormat();			// instructions format
		private var _doneBtn:TextButton;										// done button
		
		public function TextInput() {
			
			// set up format
			_format.font = "Helvetica Neue";
			_format.bold = true;
			_format.size = 13;
			_format.color = 0x000000;
			_format.align = TextFormatAlign.LEFT;
						
			// set up entry field
			_field.type = TextFieldType.INPUT;
			_field.antiAliasType = AntiAliasType.ADVANCED;
			_field.background = true;
			_field.backgroundColor = 0xdddddd;
			_field.embedFonts = true;
			_field.autoSize = TextFieldAutoSize.LEFT;
			_field.selectable = true;
			_field.wordWrap = true;
			_field.multiline = true;
			_field.text = "And if I ne|ver be|come what I am meant to be, but al|ways re|main what I am not, I shall spend e|ter|ni|ty con|tra|dic|ting my|self by be|ing at once some|thing and no|thing, a life that wants to live and is dead, and a death that wants to be dead and can|not quite a|chieve its own death be|cause it still has to ex|ist.";
			_field.width = 800;
			_field.scrollRect = new Rectangle(0, 0, 800, 575);
			_field.mouseWheelEnabled;
			_field.x = (880 - _field.width) / 2;
			_field.y = 95;
			_field.setTextFormat(_format);
			
			// set up instructions format
			_instructionsFormat.font = "Helvetica Neue";
			_instructionsFormat.size = 16;
			_instructionsFormat.color = 0xffffff;
			_instructionsFormat.align = TextFormatAlign.LEFT;
			
			// set up instructions field	
			_instructions.antiAliasType = AntiAliasType.ADVANCED;
			_instructions.embedFonts = true;
			_instructions.autoSize = TextFieldAutoSize.LEFT;
			_instructions.wordWrap = true;
			_instructions.multiline = true;
			switch (Swing.getInstance().controller) {
				
				case "wiimote":
				_instructions.text = "Enter your text in the box below. Spaces denote breaks between words; pipes ('|') denote breaks between syllables. Hit the home button on the Wiimote or click 'Accept text' when done. Hit the home button to return to this screen.";
				break;
				
				case "mouse":
				_instructions.text = "Enter your text in the box below. Spaces denote breaks between words; pipes ('|') denote breaks between syllables. Click 'Accept text' when done. Hit any key to return to this screen.";
				break;
				
			}
			_instructions.width = 640;
			_instructions.x = (880 - _field.width) / 2;
			_instructions.y = 20;
			_instructions.setTextFormat(_instructionsFormat);
			
			// set up button formatting
			var format:TextFormat = new TextFormat();
			format.font = "Helvetica Neue";
			format.bold = true;
			format.size = 16;
			format.align = TextFormatAlign.LEFT;
			format.kerning = true;
			
			// create text button
			var btn:TextButton;
			var colors:Object = {up:0x999999, over:0xffffff, highlight:0xffffff};
			_doneBtn = new TextButton("Accept text", format, colors, true);
			_doneBtn.addEventListener(MouseEvent.CLICK, handleClick, false, 0, true);
			_doneBtn.x = 690;
			_doneBtn.y = 38;
			
			// if we're in wiimote mode, listen for home button
			if (Swing.getInstance().controller == "wiimote") {
				Swing.getInstance().wiimote.addEventListener(ButtonEvent.HOME_PRESS, handleHomePress);
			}
			
			// add everything
			addChild(_field);
			addChild(_instructions);
			addChild(_doneBtn);
			
			Swing.getInstance().textLine.setText(_field.text);
					
		}
		
		/**
		 * Handles click on the done button.
		 */
		private function handleClick(event:MouseEvent):void {
			hide();
			Swing.getInstance().textLine.setText(_field.text);
		}
		
		/**
		 * Handles press of the home button.
		 */
		private function handleHomePress(event:ButtonEvent):void {
			if (visible) {
				hide();
				Swing.getInstance().textLine.setText(_field.text);
			} else {
				show();
			}
		}
		
		/**
		 * Handles key down event.
		 */
		private function handleKeyDown(event:KeyboardEvent):void {
			if (!visible) {
				show();
			}
		}
		
		/**
		 * Shows the input screen.
		 */
		public function show():void {
			visible = true;
			Swing.getInstance().textLine.hide();
		}
		
		/**
		 * Hides the input screen and sends text to the text line.
		 */
		public function hide():void {
			visible = false;
			if (!stage.hasEventListener(KeyboardEvent.KEY_DOWN)) {
				stage.addEventListener(KeyboardEvent.KEY_DOWN, handleKeyDown);
			}
			Swing.getInstance().textLine.show();
		}
	}
}