package com.erikloyer.swing
{
	import flash.display.*;
	import flash.events.Event;
	import com.erikloyer.util.*;
	import org.wiiflash.events.ButtonEvent;
	import mx.core.ButtonAsset;

	/**
	 * TextLine, Version 1.
 	 * A rotating invisible bar to which swinging letters are attached.
	 * 
	 * @author: Erik Loyer
	 * @version: 1.0.0
	 */
	public class TextLine extends Sprite
	{
		private var _length:Number;								// length of the bar
		private var _line:Shape;								// visual representation of the bar
		private var _letters:Array = new Array();				// array of hanging letters
		private var _lastPitch:Number = 0;						// last pitch value
		private var _sourceText:String;							// source text
		private var _words:Array;								// array of words from source text
		private var _wordIndex:Number = 0;						// index of the current word
		private var _syllables:Array;							// array of syllables in the current word
		private var _syllableIndex:Number = 0;					// index of the current syllable
		private var _alphaGoal:Number = 1;						// target alpha value
		
		public function TextLine(length:Number):void {
			
			_length = length;
			
			switch (Swing.getInstance().controller) {
				
				case "wiimote":
				Swing.getInstance().wiimote.addEventListener(ButtonEvent.UP_PRESS, handleUp);
				Swing.getInstance().wiimote.addEventListener(ButtonEvent.DOWN_PRESS, handleDown);
				Swing.getInstance().wiimote.addEventListener(ButtonEvent.TWO_PRESS, handleTwo);
				break;
				
				case "mouse":
				break;
				
			}
			
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			
		}
		
		/**
		 * Updates the source text.
		 *
		 * @param	text	Text to use as source.
		 */
		public function setText(text:String):void {
			_wordIndex = 0;
			_syllableIndex = 0;
			_sourceText = text;
			_words = _sourceText.split(" ");
			_syllables = _words[_wordIndex].split("|");
			formatLetters(true);
		}
		
		/**
		 * Displays visible syllables of the current word.
		 * 
		 * @param	newWord		Are we starting a new word?
		 */
		private function formatLetters(newWord:Boolean):void {
			
			alpha = 1;
			_alphaGoal = 1;
			scaleY = scaleX = 1;
			
			var word:String;
			var j:Number;
			var i:Number;
			var n:Number;
			var muteLetter:Boolean;
			
			// if we're starting a new word, then
			if (newWord) {
				
				// remove all old letters
				n = _letters.length;
				for (i=0; i<n; i++) {
					removeChild(_letters[i]);
				}
				
				// assemble the current word, keeping track of the break between
				// the syllables of the word that are visible and those that are not
				_letters = new Array();
				word = "";
				var syllableBreakIndex:Number = 0;
				for (i=0; i<_syllables.length; i++) {
					word += _syllables[i];
					if (i <= _syllableIndex) {
						syllableBreakIndex += _syllables[i].length;
					}
				}
				
				// add the letters, hiding those we're not supposed to show yet
				var numLetters:Number = word.length;
				var letterSpacing:Number = 75;
				var letter:HangingLetter;
				var totalWidth:Number = 0;
				for (i=0; i<numLetters; i++) {
					muteLetter = false;
					if (i >= syllableBreakIndex) {
						muteLetter = true;
					}
					letter = new HangingLetter(word.charAt(i), 0xffffff, muteLetter);
					totalWidth += letter.textWidth;
					_letters.push(letter);
					addChild(letter);
				}
				
				// position the letters
				var curX:Number = - (totalWidth / 2) + (_letters[0].textWidth / 2);
				for (i=0; i<numLetters; i++) {
					letter = _letters[i];
					letter.x = curX;
					if (i < (numLetters - 1)) {
						curX += ((letter.textWidth / 2) + (_letters[i+1].textWidth / 2));
					}
				}
			
			// if we're adding a syllable to an existing word, then
			} else {
				
				// assemble the current word, keeping track of the break between
				// the syllables of the word that are visible and those that are not
				word = "";
				syllableBreakIndex = 0;
				for (i=0; i<_syllables.length; i++) {
					word += _syllables[i];
					if (i <= _syllableIndex) {
						syllableBreakIndex += _syllables[i].length;
					}
				}
				
				// update which letters should be shown
				n = _letters.length;
				for (i=0; i<n; i++) {
					muteLetter = false;
					if (i >= syllableBreakIndex) {
						muteLetter = true;
					}
					_letters[i].updateMuting(muteLetter);
				}
				
			}
			
		}
		
		/**
		 * Goes to the next syllable.
		 */
		private function incrementSyllable():void {
			
			var newWord:Boolean = false;
			
			_syllableIndex++;
			if (_syllableIndex > (_syllables.length - 1)) {
				newWord = true;
				_wordIndex++;
				if (_wordIndex > (_words.length - 1)) {
					_wordIndex = 0;
				}
				_syllables = _words[_wordIndex].split("|");
				_syllableIndex = 0;
			}
			
			formatLetters(newWord);
			
		}
		
		/**
		 * Goes to the previous syllable.
		 */
		private function decrementSyllable():void {
			
			var newWord:Boolean = false;
			
			_syllableIndex--;
			if (_syllableIndex < 0) {
				newWord = true;
				_wordIndex--;
				if (_wordIndex < 0) {
					_wordIndex = _words.length - 1;
				}
				_syllables = _words[_wordIndex].split("|");
				_syllableIndex = _syllables.length - 1;
			}
			
			formatLetters(newWord);
			
		}
		
		/**
		 * User pressed Up on the Wiimote; go to previous syllable
		 */
		private function handleUp(event:ButtonEvent):void {
			decrementSyllable();
		}
		
		/**
		 * User pressed Down on the Wiimote; go to next syllable
		 */
		private function handleDown(event:ButtonEvent):void {
			incrementSyllable();
		}
		
		/**
		 * User pressed Two on the Wiimote; fade word out
		 */
		private function handleTwo(event:ButtonEvent):void {
			_alphaGoal = 0;
		}
		
		/**
		 * Per frame actions.
		 */
		private function onEnterFrame(event:Event):void {
			
			// calculate current pitch of the bar
			var pitch:Number;
			var scaleMod:Number = 1;
			switch (Swing.getInstance().controller) {
				
				case "wiimote":
				pitch = -Geometry.getDegreesFromRadians(Swing.getInstance().wiimote.pitch);
				scaleMod = 1 - Swing.getInstance().wiimote.sensorX / 20;
				break;
				
				case "mouse":
				pitch = ((stage.mouseX / 880) * 180) - 90;
				break;
				
			}
			
			// if we've crossed the zero point with sufficient velocity, increment the syllable
			if ((((_lastPitch < 0) && (pitch >= 0)) || ((_lastPitch > 0) && (pitch <= 0))) && (Math.abs(pitch - _lastPitch) > 10)) {
				incrementSyllable();
			}
			
			// transmit current rotation and rotational acceleration to the letters
			var rotAcc:Number = pitch - _lastPitch;
			rotation += ((pitch - rotation) / 3);
			var n:Number = _letters.length;
			for (var i:Number=0; i<n; i++) {
				_letters[i].perFrame(rotAcc, rotation);
			}
			_lastPitch = pitch;

			alpha += (_alphaGoal - alpha) / 10;
			
			// adjust scale
			if (Math.abs(1 - scaleMod) < .0025) {
				scaleMod = 1;
			}
			scaleY = scaleX = scaleX * scaleMod;
			
		}
		
		public function show():void {
			visible = true;
		}
		
		public function hide():void {
			visible = false;
		}
		
	}
}